@@ -23,8 +23,9 @@ class ArkoseContainer:
     subnet = None
     config = None
 
-    def __init__(self, fstype = "ext4", fssize = None, network = "none", xserver = "none", dbus = "none", dbusproxy = [], pulseaudio = False, devices = [], ctype = "cow", bind = [], restrict = [], cow = []):
+    def __init__(self, path = None, fstype = "ext4", fssize = None, network = "none", xserver = "none", dbus = "none", dbusproxy = [], pulseaudio = False, devices = [], ctype = "cow", root = "/", bind = [], restrict = [], cow = []):
         """
+            path: Base path used by Arkose to store the containers (default is ~/.arkose)
             fstype: ext4 or tmpfs
             fssize: size in MB. Defaults to 2GB for ext4 and 50% of available RAM for tmpfs
             network: Network access: "none", "direct" or "filtered"
@@ -34,6 +35,7 @@ class ArkoseContainer:
             pulseaudio: Enable pulseaudio access in the container
             devices: Allow the given devices in the container
             ctype: Container type, either cow (copy on write) or bind (full bind-mount of the FS)
+            root: Root to use for the container (default is /)
             bind: List of paths to bind-mount in the container
             restrict: List of paths to empty in the container (bind-mounts an empty directory)
             cow: List of paths to mount as copy-on-write
@@ -50,6 +52,13 @@ class ArkoseContainer:
                 network = "none"
 
         # Input validation
+        if path:
+            if type(path) != str:
+                raise Exception("path must be a string")
+            else:
+                if not os.path.exists(path):
+                    raise Exception("path '%s' doesn't exist" % path)
+
         if fstype not in ("ext4", "tmpfs"):
             raise Exception("Unsupported fstype: %s" % fstype)
 
@@ -77,6 +86,11 @@ class ArkoseContainer:
         if ctype not in ("cow", "bind"):
             raise Exception("Unsupported container type: %s" % ctype)
 
+        if root and type(root) != str:
+            raise Exception("root must be a string")
+        if not os.path.exists(root):
+            raise Exception("root must be a valid path")
+
         if type(bind) != list:
             raise Exception("bind must be a list")
 
@@ -86,6 +100,15 @@ class ArkoseContainer:
         if type(cow) != list:
             raise Exception("cow must be a list")
 
+
+        # Deal with storage path
+        if not path:
+            if not os.path.exists(os.path.expanduser("~/.arkose/")):
+                os.mkdir(os.path.expanduser("~/.arkose/"))
+            path = os.path.expanduser("~/.arkose/")
+        path = tempfile.mkdtemp(dir=path)
+        os.chmod(path, 0o755)
+
         # Set default fssize if not set by the user
         if not fssize:
             if fstype == "ext4":
@@ -94,6 +117,7 @@ class ArkoseContainer:
                 fssize = self.__default_memory()
 
         # Export all the parameters as properties
+        self.path = path
         self.fstype = fstype
         self.fssize = fssize
         self.network = network
@@ -103,6 +127,7 @@ class ArkoseContainer:
         self.pulseaudio = pulseaudio
         self.devices = devices
         self.ctype = ctype
+        self.root = root
         self.bind = bind
         self.restrict = restrict
         self.cow = cow
@@ -129,11 +154,6 @@ class ArkoseContainer:
             Initializes the filesystem and sets self.path accordingly
         """
 
-        # Create the temporary directory for arkose
-        if not os.path.exists(os.path.expanduser("~/.arkose/")):
-            os.mkdir(os.path.expanduser("~/.arkose/"))
-        self.path = tempfile.mkdtemp(dir=os.path.expanduser("~/.arkose/"))
-        os.chmod(self.path, 0o755)
         self.name = "arkose-%s" % os.path.split(self.path)[1]
         workdir = os.path.join(self.path, "workdir")
         os.mkdir(workdir)
@@ -181,11 +201,17 @@ class ArkoseContainer:
             # Mount copy-on-write overlay
             root_cow = os.path.join(workdir, "root_cow")
             os.mkdir(root_cow)
-            self.__mount("none", target, "aufs", "br=%s:/" % root_cow)
+            try:
+                try:
+                    self.__mount("none", target, "overlayfs", "upperdir=%s,lowerdir=%s" % (root_cow, self.root))
+                except:
+                    self.__mount("none", target, "aufs", "br=%s:%s" % (root_cow, self.root))
+            except:
+                raise Exception("No copy-on-write filesystem available")
 
         elif self.ctype == "bind":
             # Bind mount root device
-            self.__mount("/", target, "none", "bind,ro")
+            self.__mount(self.root, target, "none", "bind,ro")
 
         # Mount an empty directory on top of home unless it's already defined
         if not self.user.pw_dir in self.cow+self.bind+self.restrict:
@@ -211,6 +237,13 @@ class ArkoseContainer:
             os.chown(homedir_target, self.user.pw_uid, self.user.pw_gid)
             os.chmod(homedir_target, 0o700)
 
+        # Make DNS resolving work when using resolvconf
+        if os.path.exists("/run/resolvconf/"):
+            if self.ctype == "cow":
+                self.cow.append("/run/resolvconf")
+            elif self.ctype == "bind":
+                self.bind.append("/run/resolvconf")
+
         # Setup other copy-on-write mounts
         for mount in self.cow:
             # If source doesn't exist, create it
@@ -228,7 +261,13 @@ class ArkoseContainer:
             new_target = os.path.join(target, mount)
             if not os.path.exists(new_target):
                 os.mkdir(new_target)
-            self.__mount("none", new_target, "aufs", "br=%s:/%s" % (new_cow, mount))
+            try:
+                try:
+                    self.__mount("none", new_target, "overlayfs", "upperdir=%s,lowerdir=/%s" % (new_cow, mount))
+                except:
+                    self.__mount("none", new_target, "aufs", "br=%s:/%s" % (new_cow, mount))
+            except:
+                raise Exception("No copy-on-write filesystem available")
 
         # Setup other bind mounts
         for mount in self.bind:
@@ -250,9 +289,6 @@ class ArkoseContainer:
                         os.mkdir(os.path.dirname(new_target))
                     open(new_target, 'w').close()
             self.__mount("/%s" % mount, new_target, "none", "bind")
-            if os.path.isdir("/%s" % mount):
-                os.chmod(new_target, 0o700)
-                os.chown(new_target, 1000, 1000)
 
         # Setup other restricted mounts
         for mount in self.restrict:
@@ -281,6 +317,7 @@ class ArkoseContainer:
             display = self.olddisplay.split(":")
 
         if self.xserver == "none":
+            os.unsetenv("DISPLAY")
             return
 
         # Make sure we have some kind of X server
@@ -605,7 +642,7 @@ iptables -t nat -D POSTROUTING -s %s -j MASQUERADE
 lxc.network.type = veth
 lxc.network.flags = up
 lxc.network.veth.pair = %s
-lxc.network.ipv4 = %s
+lxc.network.ipv4 = %s/31
 lxc.network.script.up = %s
 """ % (interface, self.subnet[1], netup_path))
 
